# Window.py -- Pamhyr
# Copyright (C) 2023  INRAE
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-

from tools import timer, trace

from View.Tools.PamhyrWindow import PamhyrWindow
from View.Tools.PamhyrDelegate import PamhyrExTimeDelegate

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel, QCoreApplication,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QHeaderView,
)

from View.Tools.Plot.PamhyrCanvas import MplCanvas
from View.Tools.Plot.PamhyrToolbar import PamhyrPlotToolbar

from View.LateralContribution.Edit.translate import LCETranslate
from View.LateralContribution.Edit.Table import TableModel
from View.LateralContribution.Edit.Plot import Plot

_translate = QCoreApplication.translate


class EditLateralContributionWindow(PamhyrWindow):
    _pamhyr_ui = "EditLateralContribution"
    _pamhyr_name = "Edit lateral contribution"

    def __init__(self, data=None,
                 study=None, config=None,
                 parent=None):
        self._data = data
        trad = LCETranslate()

        name = self._pamhyr_name
        if self._data is not None:
            edge_name = (self._data.edge.name if self._data.edge is not None
                         else _translate("LateralContribution",
                                         "Not associate"))
            name = (
                _translate("Edit lateral contribution", self._pamhyr_name) +
                f" - {study.name} " +
                f" - {self._data.name} ({self._data.id}) " +
                f"({trad.get_dict('long_types')[self._data.lctype]} - " +
                f"{edge_name})"
            )

        super(EditLateralContributionWindow, self).__init__(
            title=name,
            study=study,
            config=config,
            trad=trad,
            parent=parent
        )

        self._hash_data.append(data)

        self.setup_table()
        self.setup_plot()
        self.setup_connections()

    def setup_table(self):
        self._delegate_time = PamhyrExTimeDelegate(
            data=self._data,
            mode=self._study.time_system,
            parent=self
        )

        headers = {}
        table_headers = self._trad.get_dict("table_headers")
        for h in self._data.header:
            headers[h] = table_headers[h]

        table = self.find(QTableView, "tableView")
        self._table = TableModel(
            table_view=table,
            table_headers=headers,
            editable_headers=self._data.header,
            delegates={
                #"time": self._delegate_time,
            },
            data=self._data,
            undo=self._undo_stack,
            trad=self._trad,
            opt_data=self._study.time_system
        )

        table.setModel(self._table)
        table.setSelectionBehavior(QAbstractItemView.SelectRows)
        table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        table.setAlternatingRowColors(True)

    def setup_plot(self):
        self.canvas = MplCanvas(width=5, height=4, dpi=100)
        self.canvas.setObjectName("canvas")
        self.toolbar = PamhyrPlotToolbar(
            self.canvas, self
        )
        self.verticalLayout.addWidget(self.toolbar)
        self.verticalLayout.addWidget(self.canvas)

        self.plot = Plot(
            canvas=self.canvas,
            data=self._data,
            mode=self._study.time_system,
            toolbar=self.toolbar,
            trad=self._trad,
            parent=self
        )
        self.plot.draw()

    def setup_connections(self):
        self.find(QAction, "action_add").triggered.connect(self.add)
        self.find(QAction, "action_del").triggered.connect(self.delete)
        self.find(QAction, "action_sort").triggered.connect(self.sort)

        self._table.dataChanged.connect(self.update)

    def update(self):
        self.plot.update()

    def index_selected_row(self):
        table = self.find(QTableView, "tableView")
        return table.selectionModel()\
                    .selectedRows()[0]\
                    .row()

    def index_selected_rows(self):
        table = self.find(QTableView, "tableView")
        return list(
            # Delete duplicate
            set(
                map(
                    lambda i: i.row(),
                    table.selectedIndexes()
                )
            )
        )

    def add(self):
        rows = self.index_selected_rows()
        if len(self._data) == 0 or len(rows) == 0:
            self._table.add(0)
        else:
            self._table.add(rows[0])

        self.plot.update()

    def delete(self):
        rows = self.index_selected_rows()
        if len(rows) == 0:
            return

        self._table.delete(rows)
        self.plot.update()

    def sort(self):
        self._table.sort(False)
        self.plot.update()

    def move_up(self):
        row = self.index_selected_row()
        self._table.move_up(row)
        self.plot.update()

    def move_down(self):
        row = self.index_selected_row()
        self._table.move_down(row)
        self.plot.update()

    def _copy(self):
        rows = self.index_selected_rows()

        table = []
        table.append(self._data.header)

        data = self._data.data
        for row in rows:
            table.append(list(data[row]))

        self.copyTableIntoClipboard(table)

    def _paste(self):
        header, data = self.parseClipboardTable()

        if len(data) == 0:
            return

        row = 0
        rows = self.index_selected_rows()
        if len(rows) != 0:
            row = rows[0]

        self._table.paste(row, header, data)
        self.plot.update()

    def _undo(self):
        self._table.undo()
        self.plot.update()

    def _redo(self):
        self._table.redo()
        self.plot.update()
