# PlotAC.py -- Pamhyr
# Copyright (C) 2023  INRAE
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-

from tools import timer
from View.Tools.PamhyrPlot import PamhyrPlot
from matplotlib import pyplot as plt

from PyQt5.QtCore import (
    QCoreApplication
)

_translate = QCoreApplication.translate


class PlotAC(PamhyrPlot):
    def __init__(self, canvas=None, trad=None, toolbar=None,
                 results=None, reach_id=0, profile_id=0,
                 parent=None):
        super(PlotAC, self).__init__(
            canvas=canvas,
            trad=trad,
            data=results,
            toolbar=toolbar,
            parent=parent
        )

        self._current_timestamp = max(results.get("timestamps"))
        self._current_reach_id = reach_id
        self._current_profile_id = profile_id

    @property
    def results(self):
        return self.data

    @results.setter
    def results(self, results):
        self.data = results
        self._current_timestamp = max(results.get("timestamps"))

    @timer
    def draw(self, highlight=None):
        self.canvas.axes.cla()
        self.canvas.axes.grid(color='grey', linestyle='--', linewidth=0.5)

        if self.results is None:
            return

        self.canvas.axes.set_xlabel(
            _translate("MainWindow_reach", "X (m)"),
            color='black', fontsize=11
        )
        self.canvas.axes.set_ylabel(
            _translate("MainWindow_reach", "Elevation (m)"),
            color='black', fontsize=11
        )

        reach = self.results.river.reach(self._current_reach_id)
        profile = reach.profile(self._current_profile_id)
        x = profile.geometry.get_station()
        z = profile.geometry.z()

        self.canvas.axes.set_xlim(
            left=min(x), right=max(x)
        )

        self.line_kp, = self.canvas.axes.plot(
            x, z,
            linestyle="solid",
            lw=1.8,
            color='grey',
        )

        kp = reach.geometry.get_kp()

        # Water elevation
        water_z = profile.get_ts_key(self._current_timestamp, "Z")

        self.water, = self.canvas.axes.plot(
            [min(x),  max(x)], [water_z, water_z],
            lw=1., color='b',
        )

        self.collection = self.canvas.axes.fill_between(
            x, z, water_z,
            where=z <= water_z,
            color='skyblue', alpha=0.7, interpolate=True
        )
        self.liste_chemins = self.collection.get_paths()

        self.canvas.figure.tight_layout()
        self.canvas.figure.canvas.draw_idle()
        if self.toolbar is not None:
            self.toolbar.update()

    def set_reach(self, reach_id):
        self._current_reach_id = reach_id
        self._current_profile_id = 0
        self.update()

    def set_profile(self, profile_id):
        self._current_profile_id = profile_id
        self.update()

    def set_timestamp(self, timestamp):
        self._current_timestamp = timestamp
        self.update_poly()

    def update(self):

        reach = self.results.river.reach(self._current_reach_id)
        profile = reach.profile(self._current_profile_id)
        x = profile.geometry.get_station()
        z = profile.geometry.z()

        self.line_kp.set_data(x, z)

        self.canvas.axes.set_xlim(
            left=min(x), right=max(x)
        )

        kp = reach.geometry.get_kp()

        # Water elevation
        water_z = profile.get_ts_key(self._current_timestamp, "Z")

        self.water.set_data([min(x),  max(x)], [water_z, water_z])

        self.collection.remove()
        self.collection = self.canvas.axes.fill_between(
            x, z, water_z,
            where=z <= water_z,
            color='skyblue', alpha=0.7, interpolate=True
        )
        self.canvas.figure.canvas.draw_idle()

    def update_poly(self):

        reach = self.results.river.reach(self._current_reach_id)
        profile = reach.profile(self._current_profile_id)
        x = profile.geometry.get_station()
        z = profile.geometry.z()

        # Water elevation
        water_z = profile.get_ts_key(self._current_timestamp, "Z")

        self.water.set_data([min(x),  max(x)], [water_z, water_z])

        self.collection.remove()
        self.collection = self.canvas.axes.fill_between(
            x, z, water_z,
            where=z <= water_z,
            color='skyblue', alpha=0.7, interpolate=True
        )
        self.canvas.figure.canvas.draw_idle()
